@@ -20,6 +20,7 @@ import traceback
 import urllib
 
 # Extern Imports
+import pkg_resources
 import requests
 from tornado.options import options
 import tornado.web
@@ -82,6 +83,7 @@ class AegisHandler(tornado.web.RequestHandler):
         self.tmpl['get_member_email'] = self.get_member_email
         self.tmpl['get_current_user'] = self.get_current_user
         self.tmpl['utcnow'] = datetime.datetime.utcnow()
+        self.tmpl['start_t'] = time.time()
         if self.cookie_get('session'):
             self.tmpl['session_ck'] = self.cookie_get('session')
         self.audit_session = {}
@@ -109,7 +111,12 @@ class AegisHandler(tornado.web.RequestHandler):
         super(AegisHandler, self).prepare()
         if self._parent_timer:
             aegis.stdlib.timer_stop(self.timer_obj, 'prepare')
-        if aegis.config.get('use_audit') and options.use_audit:
+        if aegis.config.exists('use_audit') and hasattr(config, 'hostnames'):
+            self.use_audit = config.hostnames[self.tmpl['host']].get('use_audit')
+        else:
+            self.use_audit = (aegis.config.exists('use_audit') and options.use_audit)
+        if self.use_audit:
+            self.set_marketing_id()
             self.audit_start()
 
     def finish(self, chunk=None):
@@ -154,7 +161,7 @@ class AegisHandler(tornado.web.RequestHandler):
 
     def on_finish(self):
         # This runs after the response has been sent to the client, intended for cleanup.
-        if aegis.config.get('use_audit') and options.use_audit:
+        if self.use_audit:
             self.audit_request_id = self.audit_finish()
 
     def debug_request(self):
@@ -243,11 +250,13 @@ class AegisHandler(tornado.web.RequestHandler):
 
     def _handle_request_exception(self, ex):
         if type(ex) in (aegis.database.PgsqlAdminShutdown, aegis.database.PgsqlOperationalError, aegis.database.MysqlOperationalError, aegis.database.MysqlInterfaceError):
+            logging.error("Error connecting to: %s", aegis.config.get('pg_database') or aegis.config.get('mysql_database'))
             logging.exception(ex)
             logging.error("Database is down. Send HTTP 503.")
             self.send_error(503)
             return
-        aegis.model.db().close()  # Closing database effectively does a transaction ROLLBACK
+        if aegis.config.get('pg_database') or aegis.config.get('mysql_database'):
+            aegis.model.db().close()  # Closing database effectively does a transaction ROLLBACK
         #self.logw(ex, "EX")
         #logging.exception(ex)
         # Remove cookie info to anonymize and make message shorter and more useful. Almost never used in debug.
@@ -323,7 +332,11 @@ class AegisHandler(tornado.web.RequestHandler):
             cookie_flags['secure'] = False
         # XXX Way to not set httponly for reading in javascript
         cookie_val = self.cookie_encode(value)
-        self.set_secure_cookie(self.cookie_name(name), cookie_val, expires_days=cookie_duration, domain=options.hostname, **cookie_flags)
+        if aegis.config.exists('cookie_domain'):
+            cookie_domain = aegis.config.get('cookie_domain')
+        else:
+            cookie_domain = options.hostname
+        self.set_secure_cookie(self.cookie_name(name), cookie_val, expires_days=cookie_duration, domain=cookie_domain, **cookie_flags)
 
     def cookie_get(self, name, cookie_duration=None):
         # Session cookie is set to 30 since browser should expire session cookie not time-limit
@@ -502,15 +515,22 @@ class AegisHandler(tornado.web.RequestHandler):
             audit_session_row = aegis.model.AuditSession.get_id(audit_session_id)
             self.audit_session['last_request_name'] = self.tmpl['request_name']
             self.audit_session['last_request_dttm'] = aegis.database.Literal('NOW()')
-            self.audit_session['session_time'] = aegis.database.Literal('UNIX_TIMESTAMP(NOW()) - UNIX_TIMESTAMP(created_dttm)')
+            if aegis.database.mysql_available:
+                self.audit_session['session_time'] = aegis.database.Literal('UNIX_TIMESTAMP(NOW()) - UNIX_TIMESTAMP(created_dttm)')
+            elif aegis.database.pgsql_available:
+                self.audit_session['session_time'] = aegis.database.Literal('EXTRACT(EPOCH FROM NOW()) - EXTRACT(EPOCH FROM create_dttm)')
             self.audit_session['request_cnt'] = aegis.database.Literal('request_cnt+1')
             self.audit_request['request_nbr'] = audit_session_row['request_cnt'] + 1
+            self.audit_session['marketing_id'] = audit_session_row['marketing_id']
         else:
             self.audit_session['marketing_id'] = aegis.stdlib.validate_int(self.get_marketing_id())
             self.audit_session['request_cnt'] = 1
             self.audit_request['request_nbr'] = 1
-            self.audit_session['view_cnt'] = self.view_ind
-            self.audit_session['api_cnt'] = self.api_ind
+            self.audit_session['view_cnt'] = 0
+            if hasattr(self, 'view_ind') and self.view_ind:
+                self.audit_session['view_cnt'] = 1
+            if hasattr(self, 'api_ind') and self.api_ind:
+                self.audit_session['api_cnt'] = 1
             self.audit_session['first_request_name'] = self.tmpl['request_name']
             self.audit_session['last_request_name'] = self.tmpl['request_name']
             self.audit_session['last_request_dttm'] = aegis.database.Literal('NOW()')
@@ -552,18 +572,19 @@ class AegisHandler(tornado.web.RequestHandler):
         # Audit Session - takes 10ms !!
         audit_session_id = self.tmpl.get('session_ck', {}).get('audit_session_id')
         #aegis.stdlib.logw(audit_session_id, "AUDIT SESSION ID")
+        self.audit_session['user_id'] = self.get_user_id()
         self.audit_session['member_id'] = self.get_member_id()
         if audit_session_id:
-            if self.view_ind:
+            if hasattr(self, 'view_ind') and self.view_ind:
                 self.audit_session['view_cnt'] = aegis.database.Literal('view_cnt+1')
-            if self.api_ind:
+            if hasattr(self, 'api_ind') and self.api_ind:
                 self.audit_session['api_cnt'] = aegis.database.Literal('api_cnt+1')
             aegis.model.AuditSession.update_columns(self.audit_session, {'audit_session_id': audit_session_id})
         # Audit Request
         request_args = {}
         if self.request.method in ('POST', 'PUT', 'PATCH') and self.request.args:
             post_secret_fields = ['password']
-            if config.is_defined('post_secret_fields') and options.post_secret_fields:
+            if aegis.config.exists('post_secret_fields') and options.post_secret_fields:
                 post_secret_fields += options.post_secret_fields
             request_args = dict(self.request.args)
             for field in post_secret_fields:
@@ -571,22 +592,23 @@ class AegisHandler(tornado.web.RequestHandler):
                     request_args[field] = 'SECRET'
             try:
                 #self.logw(request_args, "ARGS")
-                self.audit_request['formpost_tx'] = json.dumps(request_args)[:65535]
+                self.audit_request['formpost_tx'] = json.dumps(request_args, cls=aegis.stdlib.DateTimeEncoder)[:65535]
             except UnicodeDecodeError:
                 self.audit_request['formpost_tx'] = str(request_args)[:65535]
+        self.audit_request['user_id'] = self.get_user_id()
         self.audit_request['member_id'] = self.get_member_id()
         self.audit_request['marketing_id'] = aegis.stdlib.validate_int(self.get_marketing_id())
         self.audit_request['audit_session_id'] = audit_session_id
-        self.audit_request['view_ind'] = self.view_ind
-        self.audit_request['api_ind'] = self.api_ind
+        self.audit_request['view_ind'] = hasattr(self, 'view_ind') and self.view_ind
+        self.audit_request['api_ind'] = hasattr(self, 'api_ind') and self.api_ind
         self.audit_request['http_status_nbr'] = self._status_code
         # Optional application-specific columns
-        if config.is_defined('audit_request_columns'):
+        if aegis.config.exists('audit_request_columns'):
             for column in options.audit_request_columns:
                 if hasattr(self, column):
                     self.audit_request[column] = getattr(self, column)
         # Make the finish time the very last thing
-        self.audit_request['exec_time'] = int((time.time() - self.local_data['start_t']) * 1000)
+        self.audit_request['exec_time'] = int((time.time() - self.tmpl['start_t']) * 1000)
         # getting the database, memcache, render times on the timer - will be higher than during aegis.webapp.log_request because of above auditing
         timer_obj = aegis.stdlib.get_timer()
         if timer_obj and timer_obj._timers:
@@ -617,16 +639,19 @@ class AegisHandler(tornado.web.RequestHandler):
             request_body = {}
             if hasattr(self, 'json_req'):
                 request_body = self.json_req
-            #else:
+            # Needs Testing - maybe against HTML form
+            #elif self.request.body:
             #    request_body = self.request.body
+            #    aegis.stdlib.logw(request_body, "REQUEST BODY ")
             #    logging.warning("This may not always be a dictionary")
-            post_secret_fields = ['password']
-            if config.is_defined('post_secret_fields') and options.post_secret_fields:
+            post_secret_fields = ['password', 'accessToken', 'idToken']
+            if aegis.config.exists('post_secret_fields') and options.post_secret_fields:
                 post_secret_fields += options.post_secret_fields
             for field in post_secret_fields:
                 if field in request_args:
                     request_body[field] = 'SECRET'
-            # try to convert json_body to dictionary
+            # convert request_body to json for writing to db
+            request_body = json.dumps(request_body, cls=aegis.stdlib.DateTimeEncoder)
             self.audit_request_data = {'audit_request_id': audit_request_id, 'audit_session_id': audit_session_id,
                                        'request_url': self.request.uri, 'request_method': self.request.method,
                                        'request_headers': req_headers, 'request_body': request_body,
@@ -635,7 +660,7 @@ class AegisHandler(tornado.web.RequestHandler):
                                        'response_status': self._status_code, 'response_headers': resp_headers, 'response_ms': self.audit_request['exec_time']}
             if hasattr(self, 'json_resp'):
                 self.audit_request_data['response_body'] = self.json_resp
-                self.audit_request_data['response_bytes'] = len(json.dumps(self.json_resp))
+                self.audit_request_data['response_bytes'] = len(json.dumps(self.json_resp, cls=aegis.stdlib.DateTimeEncoder))
                 self.audit_request_data['response_error'] = json.dumps(self.json_resp.get('errors'), cls=aegis.stdlib.DateTimeEncoder)
             audit_request_data_id = aegis.model.AuditRequestData.insert_columns(**self.audit_request_data)
             audit_request_data = aegis.model.AuditRequestData.get_id(audit_request_data_id)
@@ -655,6 +680,14 @@ class AegisHandler(tornado.web.RequestHandler):
         if hasattr(self.tmpl, 'session_ck'):
             del self.tmpl['session_ck']
 
+    def get_marketing_id(self):
+        return self.audit_session.get('marketing_id')
+
+    def set_marketing_id(self):
+        mid = aegis.stdlib.validate_int(self.request.args.get('mid'))
+        if mid:
+            self.audit_session['marketing_id'] = mid
+
 
 class JsonRestApi(AegisHandler):
     def check_xsrf_cookie(self): pass
@@ -696,10 +729,9 @@ class JsonRestApi(AegisHandler):
                 json_req = json.loads(self.request.body.decode("utf-8"))
                 return json_req or {}
             except json.decoder.JSONDecodeError:
-                #return None  ??
                 raise tornado.web.HTTPError(401, 'Bad JSON Value')
         else:
-            return dict(self.request.arguments)
+            return dict(self.request.args)
 
     def json_debug(self, debug=True):
         if options.env == 'prod':
@@ -943,8 +975,11 @@ class AegisHydra(AegisWeb):
     @tornado.web.authenticated
     def get(self, *args):
         self.enforce_admin()
+        self.tmpl['page_title'] = 'Hydra %s' % options.header_logo
         self.tmpl['hydra_types'] = aegis.model.HydraType.scan()
-        self.tmpl['home_link'] = '/admin/hydra'
+        self.tmpl['home_link'] = '/'
+        self.tmpl['aegis_version'] = pkg_resources.require("aegis-tools")[0].version
+        self.tmpl['app_version'] = pkg_resources.require(aegis.config.get('program_name'))[0].version
         return self.render_path("hydra.html", **self.tmpl)
 
     @tornado.web.authenticated
@@ -1102,6 +1137,8 @@ class AegisBuild(AegisWeb):
         if env.endswith('-admin'):
             env = env.rsplit('-', maxsplit=1)[0]
         self.tmpl['live_build'] = aegis.model.Build.get_live_build(env)
+        self.tmpl['aegis_version'] = pkg_resources.require("aegis-tools")[0].version
+        self.tmpl['app_version'] = pkg_resources.require(aegis.config.get('program_name'))[0].version
         return self.render_path("build.html", **self.tmpl)
 
     @tornado.web.authenticated
